@@ -40,6 +40,7 @@
 #include <sys/types.h>
 #include <sys/fcntl.h>
 #include <sys/termios.h>
+#include <sys/wait.h>
 #include <dirent.h>
 #include <unistd.h>
 #include <stdio.h>
@@ -48,11 +49,11 @@
 #include <pwd.h>
 #endif
 
+#include <gio/gio.h>
 #include <glib/gi18n.h>
 #include <glib.h>
 #include <glib/gstdio.h>
-
-#include <gconf/gconf-client.h>
+#include <gtk/gtk.h>
 
 #include <libanjuta/anjuta-utils.h>
 #include <libanjuta/anjuta-debug.h>
@@ -480,6 +481,109 @@ anjuta_util_dialog_input (GtkWindow *parent, const gchar *prompt,
 	return (res == GTK_RESPONSE_OK);
 }
 
+static void
+on_install_files_done (GObject *proxy, GAsyncResult *result,
+					   gpointer user_data)
+{
+	GError *error = NULL;
+  	g_dbus_proxy_call_finish ((GDBusProxy *) proxy, result, &error);
+	if (error)
+	{
+		/*
+		  Only dbus error is handled. Rest of the errors are from packagekit
+		  which have already been notified to user by packagekit.
+		*/
+		if (error->domain == G_DBUS_ERROR)
+		{
+			const gchar *error_message = NULL;
+
+			/* Service error which implies packagekit is missing */
+			if (error->code == G_DBUS_ERROR_SERVICE_UNKNOWN)
+			{
+				error_message = _("You do not seem to have PackageKit installed. PackageKit is required for installing missing packages. Please install \"packagekit-gnome\" package from your distribution, or install the missing packages manually.");
+			}
+			/* General dbus error implies failure to call dbus method */
+			else if (error->code != G_DBUS_ERROR_NO_REPLY)
+			{
+				error_message = error->message;
+			}
+			if (error_message)
+				anjuta_util_dialog_error (NULL,
+										  _("Installation failed: %s"),
+										  error_message);
+		}
+		g_error_free (error);
+	}
+}
+
+gboolean
+anjuta_util_install_files (const gchar * const names)
+{
+	GDBusConnection * connection;
+	GDBusProxy * proxy;
+	guint32 xid = 0;
+	gchar ** pkgv;
+
+	if (!names)
+		return FALSE;
+
+	connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, NULL);
+	if (!connection)
+		return FALSE;
+
+	proxy = g_dbus_proxy_new_sync (connection,
+								   G_DBUS_PROXY_FLAGS_NONE,
+								   NULL,
+								   "org.freedesktop.PackageKit",
+								   "/org/freedesktop/PackageKit",
+								   "org.freedesktop.PackageKit.Modify",
+								   NULL,
+								   NULL);
+	if (!proxy)
+		return FALSE;
+
+	pkgv = g_strsplit (names, ", ", 0);
+	g_dbus_proxy_call (proxy, "InstallProvideFiles",
+					   g_variant_new ("(u^ass)",
+									  xid,
+									  pkgv,
+									  ""),
+					   G_DBUS_CALL_FLAGS_NONE,
+					   -1,
+					   NULL,
+					   on_install_files_done,
+					   NULL);
+	g_strfreev (pkgv);
+	return TRUE;
+}
+
+gboolean
+anjuta_util_package_is_installed (const gchar * package, gboolean show)
+{
+	gboolean installed = FALSE;
+	int status;
+	int exit_status;
+	pid_t pid;
+
+	if ((pid = fork()) == 0)
+		execlp ("pkg-config", "pkg-config", "--exists", package, NULL);
+
+	waitpid (pid, &status, 0);
+	exit_status = WEXITSTATUS (status);
+	installed = (exit_status == 0) ? TRUE : FALSE;
+	if (installed)
+		return TRUE;
+
+	if (show)
+	{
+		anjuta_util_dialog_error (NULL,
+								  _("The \"%s\" package is not installed.\n"
+									"Please install it."), package);
+	}
+
+	return FALSE;
+}
+
 gboolean
 anjuta_util_prog_is_installed (const gchar * prog, gboolean show)
 {
@@ -673,6 +777,92 @@ anjuta_util_get_real_path (const gchar *path)
 	return NULL;
 }
 
+/**
+ * anjuta_util_get_current_dir:
+ *
+ * Get current working directory, unlike g_get_current_dir, keeps symbolic links
+ * in path name.
+ *
+ * Returns: The current working directory.
+ */
+gchar*
+anjuta_util_get_current_dir (void)
+{
+	const gchar *pwd;
+	
+	pwd = g_getenv ("PWD");
+	if (pwd != NULL)
+	{
+		return g_strdup (pwd);
+	}
+	else
+	{
+		return g_get_current_dir ();
+	}
+}
+
+static gboolean
+is_valid_scheme_character (char c)
+{
+	return g_ascii_isalnum (c) || c == '+' || c == '-' || c == '.';
+}
+
+/* Following RFC 2396, valid schemes are built like:
+ *       scheme        = alpha *( alpha | digit | "+" | "-" | "." )
+ */
+static gboolean
+has_valid_scheme (const char *uri)
+{
+	const char *p;
+
+	p = uri;
+
+	if (!g_ascii_isalpha (*p))
+		return FALSE;
+
+	do {
+		p++;
+	} while (is_valid_scheme_character (*p));
+
+	return *p == ':';
+}
+
+/**
+ * anjuta_util_file_new_for_commandline_arg:
+ *
+ * @arg: URI or relative or absolute file path
+ *
+ * Create a new file corresponding to arg, unlike g_file_new_for_commandline_arg,
+ * keeps symbolic links in path name.
+ *
+ * Returns: A new GFile object
+ */
+GFile *
+anjuta_util_file_new_for_commandline_arg (const gchar *arg)
+{
+	GFile *file;
+	char *filename;
+	char *current_dir;
+
+	g_return_val_if_fail (arg != NULL, NULL);
+
+	if (g_path_is_absolute (arg))
+		return g_file_new_for_path (arg);
+
+	if (has_valid_scheme (arg))
+		return g_file_new_for_uri (arg);
+
+	current_dir = anjuta_util_get_current_dir ();
+	filename = g_build_filename (current_dir, arg, NULL);
+	g_free (current_dir);
+
+	file = g_file_new_for_path (filename);
+	g_free (filename);
+
+	return file;
+}
+
+
 /* Dedup a list of paths - duplicates are removed from the tail.
 ** Useful for deduping Recent Files and Recent Projects */
 GList*
@@ -731,14 +921,7 @@ anjuta_util_glist_strings_sort (GList * list)
 void
 anjuta_util_glist_strings_free (GList * list)
 {
-	GList *node;
-	node = list;
-	while (node)
-	{
-		if (node->data)
-			g_free (node->data);
-		node = g_list_next (node);
-	}
+	g_list_foreach (list, (GFunc) g_free, NULL);
 	g_list_free (list);
 }
 
@@ -827,8 +1010,8 @@ anjuta_util_create_dir (const gchar* path)
 				NULL, NULL);
 		if (g_file_info_get_file_type (info) != G_FILE_TYPE_DIRECTORY)
 		{
-			g_message ("Warning: %s is a file. \n \
-					It is trying to be treated as a directory.",g_file_get_path (dir));
+			g_message ("Warning: %s is a file. \n "
+					"It is trying to be treated as a directory.",g_file_get_path (dir));
 			g_object_unref (dir);
 			return FALSE;
 		}
@@ -940,7 +1123,8 @@ anjuta_util_user_shell (void)
 gchar **
 anjuta_util_user_terminal (void)
 {
-#ifndef G_OS_WIN32
+/* FIXME: GSettings */
+#if 0
 	GConfClient *client;
 	gchar *terminal = NULL;
 	gchar **argv = NULL;
@@ -1277,7 +1461,25 @@ gboolean
 anjuta_util_is_project_file (const gchar *filename)
 {
 	gsize len = strlen (filename);
-	return (len > 8) && (strcmp (filename + len - 7, ".anjuta") == 0);
+	return ((len > 8) && (strcmp (filename + len - 7, ".anjuta") == 0));
+}
+
+/**
+ * anjuta_util_is_template_file:
+ * @filename: the file name
+ *
+ * Return TRUE if the file is an template project file. It is implemented by
+ * checking only the file extension. So it does not check the existence
+ * of the file. But it is working on an URI if it does not containt a
+ * fragment.
+ *
+ * Returns: TRUE if the file is a template file, else FALSE
+ */
+gboolean
+anjuta_util_is_template_file (const gchar *filename)
+{
+	gsize len = strlen (filename);
+	return ((len > 9) && (strcmp (filename + len - 8, ".wiz.tgz") == 0));
 }
 
 /**
@@ -2214,9 +2416,51 @@ anjuta_utils_drop_get_files (GtkSelectionData *selection_data)
 
 	for (i = 0; uris[i] != NULL; i++)
 	{
-		GFile* file = g_file_new_for_commandline_arg (uris[0]);
+		GFile* file = g_file_new_for_uri (uris[i]);
 		files = g_slist_append(files, file);
 	}
 
 	return files;
 }
+
+/*
+ * anjuta_util_get_user_mail:
+ *
+ * Returns: The e-mail Address of the logged-in user. The resulting string
+ * must be free'd after use.
+ */
+gchar* 
+anjuta_util_get_user_mail()
+{
+	/* FIXME: Use libfolks or something like it to query the mail address */
+	return g_strconcat(g_get_user_name (), "@", g_get_host_name (), NULL);
+}
+
+/**
+ * anjuta_utils_clone_string_gptrarray:
+ * @source: The source GPtrArray containing items representing strings
+ *
+ * Clones the contents of source GPtrArray into a new allocated GPtrArray.
+ *
+ * Return a new allocated GPtrArray with strings g_strdup (), NULL on error.
+ * The returned array has set g_free as GDestroyNotity function, so that user
+ * should only care to g_ptr_array_unref () without freeing the strings.
+ */
+GPtrArray *
+anjuta_util_clone_string_gptrarray (const GPtrArray* source)
+{
+	gint i;
+	GPtrArray *dest;
+	
+	g_return_val_if_fail (source != NULL, NULL);
+		
+	dest = g_ptr_array_sized_new (source->len);
+	g_ptr_array_set_free_func (dest, g_free);
+
+	for (i = 0; i < source->len; i++)
+	{
+		g_ptr_array_add (dest, g_strdup (g_ptr_array_index (source, i)));
+	}
+
+	return dest;
+}
